param (
    [string]$Destination,
    [ValidateSet('local', 'global')][string]$InstallType,
    [string]$LicenseKey,
	[string]$ApmHost,
	[int]$ApmPort,
	[string]$AppName,
	[switch]$IncludeDotNetFramework,
    [switch]$ResetIIS,
	[switch]$ForceUpdate,
    [switch]$Help,
	[switch]$IgnoreFolderPermission
)

$AgentVersion = "6.3.0"

$w3svcRegistryPath = "HKLM:\SYSTEM\CurrentControlSet\Services\W3SVC"
$wasRegistryPath = "HKLM:\SYSTEM\CurrentControlSet\Services\WAS"
$environmentKey = "Environment"
$profilerId = "{9D363A5F-ED5F-4AAC-B456-75AFFA6AA0C8}"

$SupportedVersions = @('netcoreapp2.0','netcoreapp2.1','netcoreapp2.2','netcoreapp2.2','netcoreapp3.0','netcoreapp3.1','net5.0','net6.0','net7.0')
$Platforms = @('x64','x86')

[bool] $IsUpdateFound = $false

Function InstallAgent()
{
	try
	{
	    $agentPath = (Join-Path $Destination "DotNetCoreAgent")		
		$isVersionInfoExists = Test-Path (Join-Path $agentPath "version.info")

		if ($isVersionInfoExists) 
		{
			$installedAgentVersionString = Get-Content (Join-Path $agentPath "version.info")		
		
			$currentVersionNumber = $installedAgentVersionString -replace '\.',''
			$newVersionNumber = $AgentVersion -replace '\.',''
		
			if ($currentVersionNumber -lt $newVersionNumber) 
			{
				$IsUpdateFound = $true
				$userInput = "Y"
				if(-Not($ForceUpdate))
				{
					$agentMessage = "
APM Insight .NET core agent is already installed in the given location. Currently installed agent version is $installedAgentVersionString. Do you want to upgrade to $AgentVersion [Y/N]?"
					$userInput = Read-Host -Prompt $agentMessage
				}
				
				if ($userInput -eq "N" -OR $userInput -eq "n") 
				{
					Print("Setting the agent environment with the existing version $installedAgentVersionString.")
					SetEnvironmentVariables $agentPath 
				}
				elseif ($userInput -eq "Y" -OR $userInput -eq "y") 
				{
					Print("Upgrading the agent to the version $AgentVersion.")
					CopyFiles $agentPath
				}
				else
				{
					Print("Enter valid input.")
				}
				Print("The APM Insight .NET Core agent version $AgentVersion is set up successfully. Reset IIS, if you hosted the applications in IIS.")
			}
			elseif($currentVersionNumber -eq $newVersionNumber) 
			{
				Print("Same version of agent is found. Setting the agent environment for the version $installedAgentVersionString.")
				SetEnvironmentVariables $agentPath 
				Print("The APM Insight .NET Core agent version $AgentVersion is set up successfully. Reset IIS, if you hosted the applications in IIS.")
			}	
			else
			{
				Print("Higher version of agent is found. Upgrade could not be proceeded.")
			}
		}
		else
		{
			Print-Color -message "Setting up the APM Insight .NET Core agent version $AgentVersion." -Color Green	
			CopyFiles $agentPath
			Print-Color -message "The APM Insight .NET Core agent version $AgentVersion is set up successfully." -Color Green
			
			Print-Color -message "Kindly do the below steps to finish the configuration: `n  1. For IIS hosted applications, edit the below config file.  `n`t$agentPath\DotNetAgent\appfilter.config `n  2. For Non IIS hosted applications, edit the below config file. `n`t$agentPath\DotNetAgent\netcore_appfilter.config" -Color Yellow

			Print-Color -message "Reset IIS or Restart .NET Core applications/services to start monitoring." -Color Yellow
		}
	 }
	 catch [Exception]
	 {
		Write-Warning $_.Exception.Message;
	 }
}

Function Print([string]$message)
{
	Write-Host "
$message"
}

Function Print-Color([string]$message, [ConsoleColor[]]$Color)
{
	Write-Host "
$message" -ForeGround $Color
}

Function CopyFiles([string]$agentPath)
{
	$isFolderExists = Test-Path $agentPath
	if (-Not $isFolderExists) {
		New-Item $agentPath -type Directory | Out-Null
	}
	
	if(-Not($IsUpdateFound) -AND -Not($IgnoreFolderPermission))	{
		SetFolderPermissions $agentPath
	}
	$resolvedPath = (Resolve-Path -Path $agentPath).Path
	
	try
	{
		Get-ChildItem -Path $resolvedPath\NETFramework -Filter "*.dll" -Recurse | Rename-Item -NewName {$_.name -replace '.dll', '_old.dll' }
		Get-ChildItem -Path $resolvedPath\store -Filter "*.dll" -Recurse | Rename-Item -NewName {$_.name -replace '.dll', '_old.dll' }
		Get-ChildItem -Path $resolvedPath\x64 -Filter "*.dll" -Recurse | Rename-Item -NewName {$_.name -replace '.dll', '_old.dll' }
		Get-ChildItem -Path $resolvedPath\x86 -Filter "*.dll" -Recurse | Rename-Item -NewName {$_.name -replace '.dll', '_old.dll' }
	}
	catch [Exception]
	{
		Write-Warning $_.Exception.Message;
	}
		
	CopyAgentFiles $resolvedPath

	if(-Not($IsUpdateFound))
	{
		SetEnvironmentVariables $resolvedPath 
	 
		ModifyConfiguration $resolvedPath	
	}
		
	[string]$agentFile = $resolvedPath + "\store\";
	Get-ChildItem -Path $agentFile -Recurse | Unblock-File
		
	CreateVersionInfoFile $resolvedPath
}

Function CopyAgentFiles([string]$resolvedPath)
{	
	try
	{
		$additionalDepsPath = (Join-Path $resolvedPath "additionalDeps")	
		$isAdditionalDepsExists = Test-Path $additionalDepsPath
		if ($isAdditionalDepsExists)
		{
			Remove-Item $additionalDepsPath -Recurse
		}
	}
	catch [Exception]
	{
		Write-Warning $_.Exception.Message;
	}
	
	Copy-Item "$PSScriptRoot\Agent\Deps\additionalDeps" $resolvedPath -Recurse -Force
	Copy-Item "$PSScriptRoot\Agent\AgentDiagnostics" $resolvedPath -Recurse -Force
	if(-Not($IsUpdateFound))
	{
		Copy-Item "$PSScriptRoot\Agent\DotNetAgent" $resolvedPath -Recurse -Force
	}
	Copy-Item "$PSScriptRoot\Agent\packages\NETFramework" $resolvedPath -Recurse -Force
	Copy-Item "$PSScriptRoot\Agent\packages\x64" $resolvedPath -Recurse -Force
	Copy-Item "$PSScriptRoot\Agent\packages\x86" $resolvedPath -Recurse -Force
	
	foreach ($platform in $Platforms) {
		foreach ($folderName in $SupportedVersions) {	
			if (!(Test-Path $resolvedPath\store\$platform\$folderName\dotnetagent\1.0.0\lib\netcoreapp2.0)) {
				New-Item $resolvedPath\store\$platform\$folderName\dotnetagent\1.0.0\lib\netcoreapp2.0 -type Directory | Out-Null
			}
			Copy-Item "$PSScriptRoot\Agent\packages\AnyCPU\DotNetAgent.dll" $resolvedPath\store\$platform\$folderName\dotnetagent\1.0.0\lib\netcoreapp2.0 -Force
			
			if (!(Test-Path $resolvedPath\store\$platform\$folderName\dotnetagent.util\1.0.0\lib\netcoreapp2.0)) {
				New-Item $resolvedPath\store\$platform\$folderName\dotnetagent.util\1.0.0\lib\netcoreapp2.0 -type Directory | Out-Null
			}
			Copy-Item "$PSScriptRoot\Agent\packages\AnyCPU\DotNetAgent.Util.dll" $resolvedPath\store\$platform\$folderName\dotnetagent.util\1.0.0\lib\netcoreapp2.0 -Force
			
			if (!(Test-Path $resolvedPath\store\$platform\$folderName\site24x7.agent.api\1.0.0\lib\netcoreapp2.0)) {
				New-Item $resolvedPath\store\$platform\$folderName\site24x7.agent.api\1.0.0\lib\netcoreapp2.0 -type Directory | Out-Null
			}
			Copy-Item "$PSScriptRoot\Agent\packages\AnyCPU\Site24x7.Agent.Api.dll" $resolvedPath\store\$platform\$folderName\site24x7.agent.api\1.0.0\lib\netcoreapp2.0 -Force
			
			if (!(Test-Path $resolvedPath\store\$platform\$folderName\autoupdate\1.0.0\lib\netcoreapp2.0)) {
				New-Item $resolvedPath\store\$platform\$folderName\autoupdate\1.0.0\lib\netcoreapp2.0 -type Directory | Out-Null
			}
			Copy-Item "$PSScriptRoot\Agent\packages\AnyCPU\AutoUpdate.dll" $resolvedPath\store\$platform\$folderName\autoupdate\1.0.0\lib\netcoreapp2.0 -Force
		}
	}
}

Function ModifyConfiguration([string]$agentPath)
{
    [string]$filePath = $agentPath + "\DotNetAgent\apminsight.conf";
	if (Test-Path -Path $filePath) {
		(Get-Content $filePath ).Replace('license.key=','license.key='+$LicenseKey).Replace('apm.host=','apm.host='+$ApmHost).Replace('apm.port=','apm.port='+$ApmPort) | Out-File $filePath
	}
}

Function SetEnvironmentVariables([string]$installPath) {
	if ($InstallType -eq "global") {
		SetLocalEnvironment($installPath);
		SetGlobalEnvironment($installPath);
	}
	else {
		SetLocalEnvironment($installPath);
			
		if ((Test-Path -Path $w3svcRegistryPath) -and (Test-Path -Path $wasRegistryPath)) {
			#Setting Profiler Environment in registry for IIS starts.
			$environmentValue = GetEnvironmentSettingValue($installPath);
			SetRegistryEnvironment $w3svcRegistryPath $environmentKey $environmentValue $installPath
			SetRegistryEnvironment $wasRegistryPath $environmentKey $environmentValue $installPath
			#Setting Profiler Environment in registry for IIS ends.
		}
	}
}

Function SetLocalEnvironment([string]$installPath)
{
	if($IncludeDotNetFramework)
	{
		[Environment]::SetEnvironmentVariable("COR_ENABLE_PROFILING", "1")
		[Environment]::SetEnvironmentVariable("COR_PROFILER", $profilerId)
		[Environment]::SetEnvironmentVariable("COR_PROFILER_PATH_64", (Join-Path $installPath "NETFramework\x64\ClrProfilerAgent.dll"))
		[Environment]::SetEnvironmentVariable("COR_PROFILER_PATH_32", (Join-Path $installPath "NETFramework\x86\ClrProfilerAgent.dll"))	
	}
	[Environment]::SetEnvironmentVariable("CORECLR_ENABLE_PROFILING", "1")
	[Environment]::SetEnvironmentVariable("CORECLR_PROFILER", $profilerId)
	[Environment]::SetEnvironmentVariable("CORECLR_SITE24X7_HOME", $installPath)
	[Environment]::SetEnvironmentVariable("CORECLR_PROFILER_PATH_64", (Join-Path $installPath "x64\ClrProfilerAgent.dll"))
	[Environment]::SetEnvironmentVariable("CORECLR_PROFILER_PATH_32", (Join-Path $installPath "x86\ClrProfilerAgent.dll"))
	[Environment]::SetEnvironmentVariable("DOTNET_ADDITIONAL_DEPS", (Join-Path $installPath "additionalDeps;"))
	[Environment]::SetEnvironmentVariable("DOTNET_SHARED_STORE", (Join-Path $installPath "store;"))
	[Environment]::SetEnvironmentVariable("S247_LICENSE_KEY", $LicenseKey)
}

Function SetGlobalEnvironment([string]$installPath)
{
	if($IncludeDotNetFramework)
	{
		[Environment]::SetEnvironmentVariable("COR_ENABLE_PROFILING", "1", "Machine")
		[Environment]::SetEnvironmentVariable("COR_PROFILER", $profilerId, "Machine")
		[Environment]::SetEnvironmentVariable("COR_PROFILER_PATH_64", (Join-Path $installPath "NETFramework\x64\ClrProfilerAgent.dll"), "Machine")
		[Environment]::SetEnvironmentVariable("COR_PROFILER_PATH_32", (Join-Path $installPath "NETFramework\x86\ClrProfilerAgent.dll"), "Machine")
	}
	[Environment]::SetEnvironmentVariable("CORECLR_ENABLE_PROFILING", "1", "Machine")
	[Environment]::SetEnvironmentVariable("CORECLR_PROFILER", $profilerId, "Machine")
	[Environment]::SetEnvironmentVariable("CORECLR_SITE24X7_HOME", $installPath, "Machine")		
	[Environment]::SetEnvironmentVariable("CORECLR_PROFILER_PATH_64", (Join-Path $installPath "x64\ClrProfilerAgent.dll"), "Machine")
	[Environment]::SetEnvironmentVariable("CORECLR_PROFILER_PATH_32", (Join-Path $installPath "x86\ClrProfilerAgent.dll"), "Machine")
	[Environment]::SetEnvironmentVariable("DOTNET_ADDITIONAL_DEPS", (Join-Path $installPath "additionalDeps;"), "Machine")
	[Environment]::SetEnvironmentVariable("DOTNET_SHARED_STORE", (Join-Path $installPath "store;"), "Machine")
	[Environment]::SetEnvironmentVariable("S247_LICENSE_KEY", $LicenseKey, "Machine")
}

Function SetRegistryEnvironment([string]$registryPath, [string]$name, [string[]]$value, [string]$installPath)
{
	$exists = 0;
	
	try {
		Get-ItemProperty -Path $registryPath | Select-Object -ExpandProperty $name -ErrorAction Stop | Out-Null	
		$exists = 1;
	}
	catch {
		$exists = 0;
	}

	if ($exists -eq 0) {
		CreateNewRegistryItem $registryPath $name $value
	}
	else {
		try {
			$environmentString = (Get-ItemProperty -Path $registryPath -Name $name).Environment;
			$newEnvironmentValue = @();
			ForEach ($line in $($environmentString -split "`r`n")) {
				if($line.Contains("CORECLR_ENABLE_PROFILING=") -Or $line.Contains("CORECLR_PROFILER=") -Or $line.Contains("CORECLR_SITE24X7_HOME=") -Or $line.Contains("CORECLR_PROFILER_PATH_64=") -Or $line.Contains("CORECLR_PROFILER_PATH_32=") -Or $line.Contains("DOTNET_ADDITIONAL_DEPS=") -Or $line.Contains("DOTNET_SHARED_STORE=") -Or $line.Contains("S247_LICENSE_KEY=") -Or 
				($IncludeDotNetFramework -And ($line.Contains("COR_ENABLE_PROFILING=") -Or $line.Contains("COR_PROFILER=") -Or $line.Contains("COR_PROFILER_PATH_64=") -Or $line.Contains("COR_PROFILER_PATH_32=")))) {
				}
				else {
					$newEnvironmentValue += $line; 
				}
			}
			
			$newEnvironmentValue += $value;
			
			CreateNewRegistryItem $registryPath $name $newEnvironmentValue
		}
		catch {
			WRITE-WARNING "Failed modifying environment variable. Please contact support."
		}
	}
}

Function GetEnvironmentSettingValue([string]$installPath) 
{
	$environmentVal = @();
	if($IncludeDotNetFramework)
	{
		$environmentVal += "COR_ENABLE_PROFILING=1";
		$environmentVal += ("COR_PROFILER=" + $profilerId);
		$environmentVal += ("COR_PROFILER_PATH_64=" + (Join-Path $installPath "NETFramework\x64\ClrProfilerAgent.dll"));
		$environmentVal += ("COR_PROFILER_PATH_32=" + (Join-Path $installPath "NETFramework\x86\ClrProfilerAgent.dll"));
	}
	$environmentVal += "CORECLR_ENABLE_PROFILING=1";
	$environmentVal += ("CORECLR_PROFILER=" + $profilerId); 
	$environmentVal += ("CORECLR_SITE24X7_HOME=" + $installPath);
	$environmentVal += ("CORECLR_PROFILER_PATH_64=" + (Join-Path $installPath "x64\ClrProfilerAgent.dll")); 
	$environmentVal += ("CORECLR_PROFILER_PATH_32=" + (Join-Path $installPath "x86\ClrProfilerAgent.dll")); 
	$environmentVal += ("DOTNET_ADDITIONAL_DEPS=" + (Join-Path $installPath "additionalDeps;")); 
	$environmentVal += ("DOTNET_SHARED_STORE=" + (Join-Path $installPath "store;"));
	$environmentVal += ("S247_LICENSE_KEY=" + $LicenseKey);
	return $environmentVal;
}

Function CreateNewRegistryItem([string]$registryPath, [string]$name, [string[]]$newRegistryValue) 
{
	New-ItemProperty -Path $registryPath `
		-Name $name `
		-Value $newRegistryValue `
		-PropertyType MultiString `
		-Force | Out-Null
}

Function IsAdmin() 
{
    ([Security.Principal.WindowsPrincipal] [Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole([Security.Principal.WindowsBuiltInRole] "Administrator")
}

Function ValidateParameters() 
{
    if ($InstallType) { $InstallType = $InstallType.ToLower().Trim()}
    if ($Destination) { $Destination = $Destination.Trim()}
    if ($LicenseKey) { $LicenseKey = $LicenseKey.Trim()}
    if ($AppName) { $AppName = $AppName.Trim() }
	if ($ApmHost) { $ApmHost = $ApmHost.Trim() }

    [bool] $flag = $false
	$Tab = [char]9
    If (-Not($Destination)) {
        Print-Color -message "$Tab * Please provide -Destination parameter." -Color Yellow
		$flag = $true
    }
    
    If (-Not($InstallType)) {
        Print-Color -message  "$Tab * Please provide -InstallType parameter as global or local." -Color Yellow
		$flag = $true
    }
    
    If (-Not($LicenseKey)) {
        Print-Color -message  "$Tab * Please provide -LicenseKey parameter." -Color Yellow
		$flag = $true
    }
	
	if($flag)
	{
		exit
	}
}

If ($Help) {
    Get-Content "InstallAgentHelp.txt"
    exit
}

Function CreateVersionInfoFile([string]$agentPath)
{
	$AgentVersion | Set-Content (Join-Path $agentPath "version.info")
}

Function CheckAdminRights() {
    If (-Not(IsAdmin) -And ($installType -eq "global")) {
        Write-Host "You must have administrator rights to install the agent globally. Please run this script from an elevated shell."
        exit
    }
    
    If (-Not(IsAdmin) -And $ResetIIS) {
        Write-Host "You must have administrator rights to perform an IISReset. Please run this script from an elevated shell."
        exit
    }
}

Function SetFolderPermissions($directory) {
	try
	{
		$acl = (Get-Item $directory).GetAccessControl('Access')
		$rule1 = New-Object System.Security.AccessControl.FileSystemAccessRule("IIS_IUSRS", "FullControl", "ContainerInherit, ObjectInherit", "None", "Allow")
		$acl.AddAccessRule($rule1)
		$rule2 = New-Object System.Security.AccessControl.FileSystemAccessRule("BUILTIN\Administrators", "FullControl", "ContainerInherit, ObjectInherit", "None", "Allow")
		$acl.AddAccessRule($rule2)
		$rule3 = New-Object System.Security.AccessControl.FileSystemAccessRule([System.Security.Principal.WindowsIdentity]::GetCurrent().Name, "FullControl", "ContainerInherit, ObjectInherit", "None", "Allow")
		$acl.AddAccessRule($rule3)
		$acl.SetAccessRuleProtection($True,$False)
		Set-Acl $directory $acl
	}
	catch {
		WRITE-WARNING "Failed setting folder permission. Kindly ignore."
	}
}

ValidateParameters
CheckAdminRights
InstallAgent

# SIG # Begin signature block
# MIIl6wYJKoZIhvcNAQcCoIIl3DCCJdgCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCBUpdI77xjKO9ZI
# nt5jDps9BNdjl6+iSSW4Sn+vGT5JZaCCC30wggWAMIIEaKADAgECAhEA0Z2xpUL/
# 09mbgyCP6egP4zANBgkqhkiG9w0BAQsFADB8MQswCQYDVQQGEwJHQjEbMBkGA1UE
# CBMSR3JlYXRlciBNYW5jaGVzdGVyMRAwDgYDVQQHEwdTYWxmb3JkMRgwFgYDVQQK
# Ew9TZWN0aWdvIExpbWl0ZWQxJDAiBgNVBAMTG1NlY3RpZ28gUlNBIENvZGUgU2ln
# bmluZyBDQTAeFw0yMDEyMTAwMDAwMDBaFw0yMzEyMTAyMzU5NTlaMIHFMQswCQYD
# VQQGEwJJTjEPMA0GA1UEEQwGNjAzMjAyMRMwEQYDVQQIDApUYW1pbCBOYWR1MRUw
# EwYDVQQHDAxDaGVuZ2FscGF0dHUxIzAhBgNVBAkMGkVzdGFuY2lhIElUIFBhcmss
# IEdTVCBSb2FkMSkwJwYDVQQKDCBaT0hPIENvcnBvcmF0aW9uIFByaXZhdGUgTGlt
# aXRlZDEpMCcGA1UEAwwgWk9ITyBDb3Jwb3JhdGlvbiBQcml2YXRlIExpbWl0ZWQw
# ggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQDi3HeMGAIKgDfQ4UK+K//K
# i/IWnxPm4gsK9i3o6bX1nzqZOaPPwA8h7nDKjdwSaP0Xk47NlABcTfakTjIUinIY
# mHlYYgRafFc6+tcyP7GPJW82N3cRR0nfXygLK1A+mf0lWfwihngvwpTa4xOy+0ce
# ClOuT+38PXD4GR3MRm4tmZAmlQ1AXr4giFCtNDfcQMfIKZ5C+Tkoy13awIJsyJAJ
# uf8203YjySdS204K3GVkiRy+KhvGtQRU+eqRopWgEnSDvdz47F1QRGcDuuUVgflF
# f1SdUwXRL81ESohTlo3c4m0vWb7u/ODhggEsTOK4B61e+x7utPCquFJm8mf1oujh
# AgMBAAGjggGxMIIBrTAfBgNVHSMEGDAWgBQO4TqoUzox1Yq+wbutZxoDha00DjAd
# BgNVHQ4EFgQU+IlX0lpCD0WzABsU/rMugtg1d6owDgYDVR0PAQH/BAQDAgeAMAwG
# A1UdEwEB/wQCMAAwEwYDVR0lBAwwCgYIKwYBBQUHAwMwEQYJYIZIAYb4QgEBBAQD
# AgQQMEoGA1UdIARDMEEwNQYMKwYBBAGyMQECAQMCMCUwIwYIKwYBBQUHAgEWF2h0
# dHBzOi8vc2VjdGlnby5jb20vQ1BTMAgGBmeBDAEEATBDBgNVHR8EPDA6MDigNqA0
# hjJodHRwOi8vY3JsLnNlY3RpZ28uY29tL1NlY3RpZ29SU0FDb2RlU2lnbmluZ0NB
# LmNybDBzBggrBgEFBQcBAQRnMGUwPgYIKwYBBQUHMAKGMmh0dHA6Ly9jcnQuc2Vj
# dGlnby5jb20vU2VjdGlnb1JTQUNvZGVTaWduaW5nQ0EuY3J0MCMGCCsGAQUFBzAB
# hhdodHRwOi8vb2NzcC5zZWN0aWdvLmNvbTAfBgNVHREEGDAWgRRwcmFiaHVwQHpv
# aG9jb3JwLmNvbTANBgkqhkiG9w0BAQsFAAOCAQEAYp6kI49tFad0lkVzbeY38GTs
# AGLvoj6p57Q6mqF4bz3Igr7DeXU/sUrh2W6hSewJm/00V5uY3UQ8YtHLNki56NeF
# Ja0zISd6FJVpasL0x2+BboKiNKlogTBQO1s6a43DTrJ4cPeRafh9ZHbAM6nTarCV
# MD5RiinFNrwinIYFO65FVswVtm56+8x5vM//9RkgD3/ey3JYWZOydGbdtsKKLfqu
# Lclet9SUHiXHxWsE9fNbFWYqU+ZCfmGRJ0IROG6t88V8wszZLUDjAzLxbsZHiUq9
# dFTaD1OijtFc9IbKOVK0s0S0UeUimP+ED7o/Y1N2+8S0DmKd97pAA/1EvTK7bTCC
# BfUwggPdoAMCAQICEB2iSDBvmyYY0ILgln0z02owDQYJKoZIhvcNAQEMBQAwgYgx
# CzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpOZXcgSmVyc2V5MRQwEgYDVQQHEwtKZXJz
# ZXkgQ2l0eTEeMBwGA1UEChMVVGhlIFVTRVJUUlVTVCBOZXR3b3JrMS4wLAYDVQQD
# EyVVU0VSVHJ1c3QgUlNBIENlcnRpZmljYXRpb24gQXV0aG9yaXR5MB4XDTE4MTEw
# MjAwMDAwMFoXDTMwMTIzMTIzNTk1OVowfDELMAkGA1UEBhMCR0IxGzAZBgNVBAgT
# EkdyZWF0ZXIgTWFuY2hlc3RlcjEQMA4GA1UEBxMHU2FsZm9yZDEYMBYGA1UEChMP
# U2VjdGlnbyBMaW1pdGVkMSQwIgYDVQQDExtTZWN0aWdvIFJTQSBDb2RlIFNpZ25p
# bmcgQ0EwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCGIo0yhXoYn0nw
# li9jCB4t3HyfFM/jJrYlZilAhlRGdDFixRDtsocnppnLlTDAVvWkdcapDlBipVGR
# EGrgS2Ku/fD4GKyn/+4uMyD6DBmJqGx7rQDDYaHcaWVtH24nlteXUYam9CflfGqL
# lR5bYNV+1xaSnAAvaPeX7Wpyvjg7Y96Pv25MQV0SIAhZ6DnNj9LWzwa0VwW2TqE+
# V2sfmLzEYtYbC43HZhtKn52BxHJAteJf7wtF/6POF6YtVbC3sLxUap28jVZTxvC6
# eVBJLPcDuf4vZTXyIuosB69G2flGHNyMfHEo8/6nxhTdVZFuihEN3wYklX0Pp6F8
# OtqGNWHTAgMBAAGjggFkMIIBYDAfBgNVHSMEGDAWgBRTeb9aqitKz1SA4dibwJ3y
# sgNmyzAdBgNVHQ4EFgQUDuE6qFM6MdWKvsG7rWcaA4WtNA4wDgYDVR0PAQH/BAQD
# AgGGMBIGA1UdEwEB/wQIMAYBAf8CAQAwHQYDVR0lBBYwFAYIKwYBBQUHAwMGCCsG
# AQUFBwMIMBEGA1UdIAQKMAgwBgYEVR0gADBQBgNVHR8ESTBHMEWgQ6BBhj9odHRw
# Oi8vY3JsLnVzZXJ0cnVzdC5jb20vVVNFUlRydXN0UlNBQ2VydGlmaWNhdGlvbkF1
# dGhvcml0eS5jcmwwdgYIKwYBBQUHAQEEajBoMD8GCCsGAQUFBzAChjNodHRwOi8v
# Y3J0LnVzZXJ0cnVzdC5jb20vVVNFUlRydXN0UlNBQWRkVHJ1c3RDQS5jcnQwJQYI
# KwYBBQUHMAGGGWh0dHA6Ly9vY3NwLnVzZXJ0cnVzdC5jb20wDQYJKoZIhvcNAQEM
# BQADggIBAE1jUO1HNEphpNveaiqMm/EAAB4dYns61zLC9rPgY7P7YQCImhttEAcE
# T7646ol4IusPRuzzRl5ARokS9At3WpwqQTr81vTr5/cVlTPDoYMot94v5JT3hTOD
# LUpASL+awk9KsY8k9LOBN9O3ZLCmI2pZaFJCX/8E6+F0ZXkI9amT3mtxQJmWunjx
# ucjiwwgWsatjWsgVgG10Xkp1fqW4w2y1z99KeYdcx0BNYzX2MNPPtQoOCwR/oEuu
# u6Ol0IQAkz5TXTSlADVpbL6fICUQDRn7UJBhvjmPeo5N9p8OHv4HURJmgyYZSJXO
# SsnBf/M6BZv5b9+If8AjntIeQ3pFMcGcTanwWbJZGehqjSkEAnd8S0vNcL46slVa
# eD68u28DECV3FTSK+TbMQ5Lkuk/xYpMoJVcp+1EZx6ElQGqEV8aynbG8HArafGd+
# fS7pKEwYfsR7MUFxmksp7As9V1DSyt39ngVR5UR43QHesXWYDVQk/fBO4+L4g71y
# uss9Ou7wXheSaG3IYfmm8SoKC6W59J7umDIFhZ7r+YMp08Ysfb06dy6LN0KgaoLt
# O0qqlBCk4Q34F8W2WnkzGJLjtXX4oemOCiUe5B7xn1qHI/+fpFGe+zmAEc3btcSn
# qIBv5VPU4OOiwtJbGvoyJi1qV3AcPKRYLqPzW0sH3DJZ84enGm1YMYIZxDCCGcAC
# AQEwgZEwfDELMAkGA1UEBhMCR0IxGzAZBgNVBAgTEkdyZWF0ZXIgTWFuY2hlc3Rl
# cjEQMA4GA1UEBxMHU2FsZm9yZDEYMBYGA1UEChMPU2VjdGlnbyBMaW1pdGVkMSQw
# IgYDVQQDExtTZWN0aWdvIFJTQSBDb2RlIFNpZ25pbmcgQ0ECEQDRnbGlQv/T2ZuD
# II/p6A/jMA0GCWCGSAFlAwQCAQUAoIHAMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3
# AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMC8GCSqGSIb3DQEJBDEi
# BCAoSEzfUeEkPG23eyEDA6wk72Utu5UfRI9QQNuqpgBH6zBUBgorBgEEAYI3AgEM
# MUYwRKBCgEAAWgBPAEgATwAgAEMAbwByAHAAbwByAGEAdABpAG8AbgAgAFAAcgBp
# AHYAYQB0AGUAIABMAGkAbQBpAHQAZQBkMA0GCSqGSIb3DQEBAQUABIIBAAeR7Ps5
# 0muxVgZkdRbihFMu5CGd1l9rzNyCBai2fAnC5b8pHL/iUniTO8idQBpusDYUjps2
# I0hw9aNs9PwrtMsZeT59wsnUHX6zPipd9hKP86ddOQ1cd1w3QITcG8/fB2ZOZOLP
# v/Fe+FUKgZQ3IVCGMT/VDPzC2YiazdNd1/zsPwzDHrg/rrpr/MrwCn5VKmIpjaeq
# ChNuqdPmSfX8ab/25hwZzhxUa2ySZi6ojDZdgamU25Og/hAILoD63L4FUo1HmEcO
# xoghi9owZahgOmk+84TlQUvDM8ZyJTIvwah3FLhzs8QTH1yW/f0rxfp9orJY+Ksv
# xqtHAvGI5CX+pIOhghdAMIIXPAYKKwYBBAGCNwMDATGCFywwghcoBgkqhkiG9w0B
# BwKgghcZMIIXFQIBAzEPMA0GCWCGSAFlAwQCAQUAMHgGCyqGSIb3DQEJEAEEoGkE
# ZzBlAgEBBglghkgBhv1sBwEwMTANBglghkgBZQMEAgEFAAQg2wNZ1+PFO5Otsct/
# cMLQdWnsqbMOvJujuGBxbTSzh70CEQDzD4aEYU3UNys9txWDQ9IoGA8yMDIzMTEx
# NTA4NTcxNlqgghMJMIIGwjCCBKqgAwIBAgIQBUSv85SdCDmmv9s/X+VhFjANBgkq
# hkiG9w0BAQsFADBjMQswCQYDVQQGEwJVUzEXMBUGA1UEChMORGlnaUNlcnQsIElu
# Yy4xOzA5BgNVBAMTMkRpZ2lDZXJ0IFRydXN0ZWQgRzQgUlNBNDA5NiBTSEEyNTYg
# VGltZVN0YW1waW5nIENBMB4XDTIzMDcxNDAwMDAwMFoXDTM0MTAxMzIzNTk1OVow
# SDELMAkGA1UEBhMCVVMxFzAVBgNVBAoTDkRpZ2lDZXJ0LCBJbmMuMSAwHgYDVQQD
# ExdEaWdpQ2VydCBUaW1lc3RhbXAgMjAyMzCCAiIwDQYJKoZIhvcNAQEBBQADggIP
# ADCCAgoCggIBAKNTRYcdg45brD5UsyPgz5/X5dLnXaEOCdwvSKOXejsqnGfcYhVY
# wamTEafNqrJq3RApih5iY2nTWJw1cb86l+uUUI8cIOrHmjsvlmbjaedp/lvD1isg
# HMGXlLSlUIHyz8sHpjBoyoNC2vx/CSSUpIIa2mq62DvKXd4ZGIX7ReoNYWyd/nFe
# xAaaPPDFLnkPG2ZS48jWPl/aQ9OE9dDH9kgtXkV1lnX+3RChG4PBuOZSlbVH13gp
# OWvgeFmX40QrStWVzu8IF+qCZE3/I+PKhu60pCFkcOvV5aDaY7Mu6QXuqvYk9R28
# mxyyt1/f8O52fTGZZUdVnUokL6wrl76f5P17cz4y7lI0+9S769SgLDSb495uZBkH
# NwGRDxy1Uc2qTGaDiGhiu7xBG3gZbeTZD+BYQfvYsSzhUa+0rRUGFOpiCBPTaR58
# ZE2dD9/O0V6MqqtQFcmzyrzXxDtoRKOlO0L9c33u3Qr/eTQQfqZcClhMAD6FaXXH
# g2TWdc2PEnZWpST618RrIbroHzSYLzrqawGw9/sqhux7UjipmAmhcbJsca8+uG+W
# 1eEQE/5hRwqM/vC2x9XH3mwk8L9CgsqgcT2ckpMEtGlwJw1Pt7U20clfCKRwo+wK
# 8REuZODLIivK8SgTIUlRfgZm0zu++uuRONhRB8qUt+JQofM604qDy0B7AgMBAAGj
# ggGLMIIBhzAOBgNVHQ8BAf8EBAMCB4AwDAYDVR0TAQH/BAIwADAWBgNVHSUBAf8E
# DDAKBggrBgEFBQcDCDAgBgNVHSAEGTAXMAgGBmeBDAEEAjALBglghkgBhv1sBwEw
# HwYDVR0jBBgwFoAUuhbZbU2FL3MpdpovdYxqII+eyG8wHQYDVR0OBBYEFKW27xPn
# 783QZKHVVqllMaPe1eNJMFoGA1UdHwRTMFEwT6BNoEuGSWh0dHA6Ly9jcmwzLmRp
# Z2ljZXJ0LmNvbS9EaWdpQ2VydFRydXN0ZWRHNFJTQTQwOTZTSEEyNTZUaW1lU3Rh
# bXBpbmdDQS5jcmwwgZAGCCsGAQUFBwEBBIGDMIGAMCQGCCsGAQUFBzABhhhodHRw
# Oi8vb2NzcC5kaWdpY2VydC5jb20wWAYIKwYBBQUHMAKGTGh0dHA6Ly9jYWNlcnRz
# LmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydFRydXN0ZWRHNFJTQTQwOTZTSEEyNTZUaW1l
# U3RhbXBpbmdDQS5jcnQwDQYJKoZIhvcNAQELBQADggIBAIEa1t6gqbWYF7xwjU+K
# PGic2CX/yyzkzepdIpLsjCICqbjPgKjZ5+PF7SaCinEvGN1Ott5s1+FgnCvt7T1I
# jrhrunxdvcJhN2hJd6PrkKoS1yeF844ektrCQDifXcigLiV4JZ0qBXqEKZi2V3mP
# 2yZWK7Dzp703DNiYdk9WuVLCtp04qYHnbUFcjGnRuSvExnvPnPp44pMadqJpddNQ
# 5EQSviANnqlE0PjlSXcIWiHFtM+YlRpUurm8wWkZus8W8oM3NG6wQSbd3lqXTzON
# 1I13fXVFoaVYJmoDRd7ZULVQjK9WvUzF4UbFKNOt50MAcN7MmJ4ZiQPq1JE3701S
# 88lgIcRWR+3aEUuMMsOI5ljitts++V+wQtaP4xeR0arAVeOGv6wnLEHQmjNKqDbU
# uXKWfpd5OEhfysLcPTLfddY2Z1qJ+Panx+VPNTwAvb6cKmx5AdzaROY63jg7B145
# WPR8czFVoIARyxQMfq68/qTreWWqaNYiyjvrmoI1VygWy2nyMpqy0tg6uLFGhmu6
# F/3Ed2wVbK6rr3M66ElGt9V/zLY4wNjsHPW2obhDLN9OTH0eaHDAdwrUAuBcYLso
# /zjlUlrWrBciI0707NMX+1Br/wd3H3GXREHJuEbTbDJ8WC9nR2XlG3O2mflrLAZG
# 70Ee8PBf4NvZrZCARK+AEEGKMIIGrjCCBJagAwIBAgIQBzY3tyRUfNhHrP0oZipe
# WzANBgkqhkiG9w0BAQsFADBiMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNl
# cnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMSEwHwYDVQQDExhEaWdp
# Q2VydCBUcnVzdGVkIFJvb3QgRzQwHhcNMjIwMzIzMDAwMDAwWhcNMzcwMzIyMjM1
# OTU5WjBjMQswCQYDVQQGEwJVUzEXMBUGA1UEChMORGlnaUNlcnQsIEluYy4xOzA5
# BgNVBAMTMkRpZ2lDZXJ0IFRydXN0ZWQgRzQgUlNBNDA5NiBTSEEyNTYgVGltZVN0
# YW1waW5nIENBMIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAxoY1Bkmz
# wT1ySVFVxyUDxPKRN6mXUaHW0oPRnkyibaCwzIP5WvYRoUQVQl+kiPNo+n3znIkL
# f50fng8zH1ATCyZzlm34V6gCff1DtITaEfFzsbPuK4CEiiIY3+vaPcQXf6sZKz5C
# 3GeO6lE98NZW1OcoLevTsbV15x8GZY2UKdPZ7Gnf2ZCHRgB720RBidx8ald68Dd5
# n12sy+iEZLRS8nZH92GDGd1ftFQLIWhuNyG7QKxfst5Kfc71ORJn7w6lY2zkpsUd
# zTYNXNXmG6jBZHRAp8ByxbpOH7G1WE15/tePc5OsLDnipUjW8LAxE6lXKZYnLvWH
# po9OdhVVJnCYJn+gGkcgQ+NDY4B7dW4nJZCYOjgRs/b2nuY7W+yB3iIU2YIqx5K/
# oN7jPqJz+ucfWmyU8lKVEStYdEAoq3NDzt9KoRxrOMUp88qqlnNCaJ+2RrOdOqPV
# A+C/8KI8ykLcGEh/FDTP0kyr75s9/g64ZCr6dSgkQe1CvwWcZklSUPRR8zZJTYsg
# 0ixXNXkrqPNFYLwjjVj33GHek/45wPmyMKVM1+mYSlg+0wOI/rOP015LdhJRk8mM
# DDtbiiKowSYI+RQQEgN9XyO7ZONj4KbhPvbCdLI/Hgl27KtdRnXiYKNYCQEoAA6E
# VO7O6V3IXjASvUaetdN2udIOa5kM0jO0zbECAwEAAaOCAV0wggFZMBIGA1UdEwEB
# /wQIMAYBAf8CAQAwHQYDVR0OBBYEFLoW2W1NhS9zKXaaL3WMaiCPnshvMB8GA1Ud
# IwQYMBaAFOzX44LScV1kTN8uZz/nupiuHA9PMA4GA1UdDwEB/wQEAwIBhjATBgNV
# HSUEDDAKBggrBgEFBQcDCDB3BggrBgEFBQcBAQRrMGkwJAYIKwYBBQUHMAGGGGh0
# dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBBBggrBgEFBQcwAoY1aHR0cDovL2NhY2Vy
# dHMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0VHJ1c3RlZFJvb3RHNC5jcnQwQwYDVR0f
# BDwwOjA4oDagNIYyaHR0cDovL2NybDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0VHJ1
# c3RlZFJvb3RHNC5jcmwwIAYDVR0gBBkwFzAIBgZngQwBBAIwCwYJYIZIAYb9bAcB
# MA0GCSqGSIb3DQEBCwUAA4ICAQB9WY7Ak7ZvmKlEIgF+ZtbYIULhsBguEE0TzzBT
# zr8Y+8dQXeJLKftwig2qKWn8acHPHQfpPmDI2AvlXFvXbYf6hCAlNDFnzbYSlm/E
# UExiHQwIgqgWvalWzxVzjQEiJc6VaT9Hd/tydBTX/6tPiix6q4XNQ1/tYLaqT5Fm
# niye4Iqs5f2MvGQmh2ySvZ180HAKfO+ovHVPulr3qRCyXen/KFSJ8NWKcXZl2szw
# cqMj+sAngkSumScbqyQeJsG33irr9p6xeZmBo1aGqwpFyd/EjaDnmPv7pp1yr8TH
# wcFqcdnGE4AJxLafzYeHJLtPo0m5d2aR8XKc6UsCUqc3fpNTrDsdCEkPlM05et3/
# JWOZJyw9P2un8WbDQc1PtkCbISFA0LcTJM3cHXg65J6t5TRxktcma+Q4c6umAU+9
# Pzt4rUyt+8SVe+0KXzM5h0F4ejjpnOHdI/0dKNPH+ejxmF/7K9h+8kaddSweJywm
# 228Vex4Ziza4k9Tm8heZWcpw8De/mADfIBZPJ/tgZxahZrrdVcA6KYawmKAr7ZVB
# tzrVFZgxtGIJDwq9gdkT/r+k0fNX2bwE+oLeMt8EifAAzV3C+dAjfwAL5HYCJtnw
# ZXZCpimHCUcr5n8apIUP/JiW9lVUKx+A+sDyDivl1vupL0QVSucTDh3bNzgaoSv2
# 7dZ8/DCCBY0wggR1oAMCAQICEA6bGI750C3n79tQ4ghAGFowDQYJKoZIhvcNAQEM
# BQAwZTELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UE
# CxMQd3d3LmRpZ2ljZXJ0LmNvbTEkMCIGA1UEAxMbRGlnaUNlcnQgQXNzdXJlZCBJ
# RCBSb290IENBMB4XDTIyMDgwMTAwMDAwMFoXDTMxMTEwOTIzNTk1OVowYjELMAkG
# A1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRp
# Z2ljZXJ0LmNvbTEhMB8GA1UEAxMYRGlnaUNlcnQgVHJ1c3RlZCBSb290IEc0MIIC
# IjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAv+aQc2jeu+RdSjwwIjBpM+zC
# pyUuySE98orYWcLhKac9WKt2ms2uexuEDcQwH/MbpDgW61bGl20dq7J58soR0uRf
# 1gU8Ug9SH8aeFaV+vp+pVxZZVXKvaJNwwrK6dZlqczKU0RBEEC7fgvMHhOZ0O21x
# 4i0MG+4g1ckgHWMpLc7sXk7Ik/ghYZs06wXGXuxbGrzryc/NrDRAX7F6Zu53yEio
# ZldXn1RYjgwrt0+nMNlW7sp7XeOtyU9e5TXnMcvak17cjo+A2raRmECQecN4x7ax
# xLVqGDgDEI3Y1DekLgV9iPWCPhCRcKtVgkEy19sEcypukQF8IUzUvK4bA3VdeGbZ
# OjFEmjNAvwjXWkmkwuapoGfdpCe8oU85tRFYF/ckXEaPZPfBaYh2mHY9WV1CdoeJ
# l2l6SPDgohIbZpp0yt5LHucOY67m1O+SkjqePdwA5EUlibaaRBkrfsCUtNJhbesz
# 2cXfSwQAzH0clcOP9yGyshG3u3/y1YxwLEFgqrFjGESVGnZifvaAsPvoZKYz0YkH
# 4b235kOkGLimdwHhD5QMIR2yVCkliWzlDlJRR3S+Jqy2QXXeeqxfjT/JvNNBERJb
# 5RBQ6zHFynIWIgnffEx1P2PsIV/EIFFrb7GrhotPwtZFX50g/KEexcCPorF+CiaZ
# 9eRpL5gdLfXZqbId5RsCAwEAAaOCATowggE2MA8GA1UdEwEB/wQFMAMBAf8wHQYD
# VR0OBBYEFOzX44LScV1kTN8uZz/nupiuHA9PMB8GA1UdIwQYMBaAFEXroq/0ksuC
# MS1Ri6enIZ3zbcgPMA4GA1UdDwEB/wQEAwIBhjB5BggrBgEFBQcBAQRtMGswJAYI
# KwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBDBggrBgEFBQcwAoY3
# aHR0cDovL2NhY2VydHMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9v
# dENBLmNydDBFBgNVHR8EPjA8MDqgOKA2hjRodHRwOi8vY3JsMy5kaWdpY2VydC5j
# b20vRGlnaUNlcnRBc3N1cmVkSURSb290Q0EuY3JsMBEGA1UdIAQKMAgwBgYEVR0g
# ADANBgkqhkiG9w0BAQwFAAOCAQEAcKC/Q1xV5zhfoKN0Gz22Ftf3v1cHvZqsoYcs
# 7IVeqRq7IviHGmlUIu2kiHdtvRoU9BNKei8ttzjv9P+Aufih9/Jy3iS8UgPITtAq
# 3votVs/59PesMHqai7Je1M/RQ0SbQyHrlnKhSLSZy51PpwYDE3cnRNTnf+hZqPC/
# Lwum6fI0POz3A8eHqNJMQBk1RmppVLC4oVaO7KTVPeix3P0c2PR3WlxUjG/voVA9
# /HYJaISfb8rbII01YBwCA8sgsKxYoA5AY8WYIsGyWfVVa88nq2x2zm8jLfR+cWoj
# ayL/ErhULSd+2DrZ8LaHlv1b0VysGMNNn3O3AamfV6peKOK5lDGCA3YwggNyAgEB
# MHcwYzELMAkGA1UEBhMCVVMxFzAVBgNVBAoTDkRpZ2lDZXJ0LCBJbmMuMTswOQYD
# VQQDEzJEaWdpQ2VydCBUcnVzdGVkIEc0IFJTQTQwOTYgU0hBMjU2IFRpbWVTdGFt
# cGluZyBDQQIQBUSv85SdCDmmv9s/X+VhFjANBglghkgBZQMEAgEFAKCB0TAaBgkq
# hkiG9w0BCQMxDQYLKoZIhvcNAQkQAQQwHAYJKoZIhvcNAQkFMQ8XDTIzMTExNTA4
# NTcxNlowKwYLKoZIhvcNAQkQAgwxHDAaMBgwFgQUZvArMsLCyQ+CXc6qisnGTxmc
# z0AwLwYJKoZIhvcNAQkEMSIEIGY1Gi+8eqXr458ZjlkBSBHSC4k5dZuEG790PQ5H
# jNP0MDcGCyqGSIb3DQEJEAIvMSgwJjAkMCIEINL25G3tdCLM0dRAV2hBNm+CitpV
# mq4zFq9NGprUDHgoMA0GCSqGSIb3DQEBAQUABIICADqamY7LqXqnCRH/WwAdhRWQ
# 5ubYz6zAPNceHiuZFjrJiFymg5PvTgvN9IiOeoq8FQhqpkFcfl864kl6HENphdPY
# cVIKAkMPpJ+Q3zzyq7swT33wKj3EQ40opd+It6xlngW/N5g59ejwNAJgROlm/gq3
# 7uY8OqCJOYZEdM/0S6mIdHXBdWI09jqrF5hucZCzxY1EUmykV7NZEPb45FiFZk3s
# JlKusppUVNt3mP8en2nElne/WBacPPXtx2Bkg7vAaRW7j13HjVvUBRbKYYmoTEKC
# keXHPXhOfzNhvndWWVuTYl2UquaPWlka1FV4N18rD3vg9ug2rFIOXtIV8kSy1LEv
# 4YabPM65toMG4UjsP042P60J6tQ59C6Ury7zEEzoLN8kR18ZLAp3k8LCFyZ+O3E7
# bCQMLGZeQWmkEVl9kg0ZZt5Xo09BraOaq3rdgzGREVJouF3iyFxBTKK8SGQV8za2
# 7EsEy22dmtt6TaI3UrNB/ONo5pg2hydsg2ida6qwko5bpeNz47WRlAY/ZOXjZLj4
# Rw7Yj2oYflCZT8RMY0GS+tZTBG4Cl82DX6L3F0pZOiWm6T9wazPA08VMefxaL6uM
# dx8G4IIMamEAFAz3NHEvXJE8kxG5NZ9rWJpqCbtWKON0uumVm7+RDjmgdRd5iVr1
# TGxwvTRRRP1BRFz04b98
# SIG # End signature block
